@@ -31,20 +31,24 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <gtk/gtk.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkkeysyms.h>
-#include <games-scores.h>
-#include <games-scores-dialog.h>
-#include <games-frame.h>
-#include <games-files.h>
-#include <games-gridframe.h>
-#include <games-preimage.h>
-#include <games-stock.h>
-#include <games-conf.h>
 
-#ifdef HAVE_GNOME
-#include <gnome.h>
-#endif
+#include <libgames-support/games-conf.h>
+#include <libgames-support/games-files.h>
+#include <libgames-support/games-frame.h>
+#include <libgames-support/games-gridframe.h>
+#include <libgames-support/games-help.h>
+#include <libgames-support/games-preimage.h>
+#include <libgames-support/games-runtime.h>
+#include <libgames-support/games-scores.h>
+#include <libgames-support/games-scores-dialog.h>
+#include <libgames-support/games-stock.h>
+
+#ifdef WITH_SMCLIENT
+#include <libgames-support/eggsmclient.h>
+#endif /* WITH_SMCLIENT */
 
 #include "glines.h"
 
@@ -160,7 +164,7 @@ static char *ball_filename;
 static GtkWidget *scorelabel;
 static scoretable sctab[] =
   { {5, 10}, {6, 12}, {7, 18}, {8, 28}, {9, 42}, {10, 82}, {11, 108}, {12,
-								       138},
+                                                                       138},
   {13, 172}, {14, 210}, {0, 0} };
 
 static struct {
@@ -176,7 +180,7 @@ set_statusbar_message (gchar * message)
 {
   guint context_id;
   context_id = gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar),
-					     "message");
+                                             "message");
   gtk_statusbar_pop (GTK_STATUSBAR (statusbar), context_id);
   gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, message);
 }
@@ -188,21 +192,21 @@ show_image_warning (gchar * message)
   GtkWidget *button;
 
   dialog = gtk_message_dialog_new (GTK_WINDOW (app),
-				   GTK_DIALOG_MODAL,
-				   GTK_MESSAGE_WARNING,
-				   GTK_BUTTONS_CLOSE,
-				   _("Could not load theme"));
+                                   GTK_DIALOG_MODAL,
+                                   GTK_MESSAGE_WARNING,
+                                   GTK_BUTTONS_CLOSE,
+                                   "%s",_("Could not load theme"));
 
   gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-					    message);
+                                            "%s", message);
 
   button = gtk_dialog_add_button (GTK_DIALOG (dialog),
-				  _("Preferences"), GTK_RESPONSE_ACCEPT);
+                                  _("Preferences"), GTK_RESPONSE_ACCEPT);
 
   gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
 
   g_signal_connect (button, "clicked", G_CALLBACK (game_props_callback),
-		    NULL);
+                    NULL);
 
   gtk_dialog_run (GTK_DIALOG (dialog));
   gtk_widget_destroy (dialog);
@@ -213,20 +217,22 @@ load_image (gchar * fname)
 {
   GamesPreimage *preimage;
   gchar *path;
+  const char *dirname;
   GError *error = NULL;
 
-  path = g_build_filename (PIXMAPDIR, fname, NULL);
+  dirname = games_runtime_get_directory (GAMES_RUNTIME_GAME_PIXMAP_DIRECTORY);
+  path = g_build_filename (dirname, fname, NULL);
   if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
     warning_message = g_strdup_printf (_("Unable to locate file:\n%s\n\n"
-					 "The default theme will be loaded instead."),
-				       fname);
+                                         "The default theme will be loaded instead."),
+                                       fname);
 
-    path = g_build_filename (PIXMAPDIR, "balls.svg", NULL);
+    path = g_build_filename (dirname, "balls.svg", NULL);
     if (!g_file_test (path, G_FILE_TEST_EXISTS)) {
       g_free (warning_message);
       warning_message = g_strdup_printf (_("Unable to locate file:\n%s\n\n"
-					   "Please check that Five or More is installed correctly."),
-					 fname);
+                                           "Please check that Five or More is installed correctly."),
+                                         fname);
     }
   }
 
@@ -255,7 +261,7 @@ relay_table (void)
   g_value_set_int (&value, height);
 
   gtk_container_child_set_property (GTK_CONTAINER (table),
-				    bottom_pane, "bottom_attach", &value);
+                                    bottom_pane, "bottom_attach", &value);
 
   gtk_table_resize (GTK_TABLE (table), vfieldsize + 1, 1);
 }
@@ -274,7 +280,7 @@ refresh_pixmaps (void)
 
   if (ball_preimage) {
     ball_pixbuf = games_preimage_render (ball_preimage, 4 * boxsize,
-					 7 * boxsize);
+                                         7 * boxsize);
 
     /* Handle rendering problems. */
     if (!ball_pixbuf) {
@@ -282,15 +288,15 @@ refresh_pixmaps (void)
       ball_preimage = NULL;
 
       if (!warning_message) {
-	warning_message = g_strdup ("The selected theme failed to render.\n\n"
-				    "Please check that Five or More is installed correctly.");
+        warning_message = g_strdup ("The selected theme failed to render.\n\n"
+                                    "Please check that Five or More is installed correctly.");
       }
     }
   }
 
   if (!ball_pixbuf) {
     ball_pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-				  4 * boxsize, 7 * boxsize);
+                                  4 * boxsize, 7 * boxsize);
     gdk_pixbuf_fill (ball_pixbuf, 0x00000000);
   }
 
@@ -306,8 +312,8 @@ refresh_pixmaps (void)
   gdk_draw_rectangle (ball_pixmap, gc, TRUE, 0, 0, 4 * boxsize, 7 * boxsize);
 
   gdk_draw_pixbuf (ball_pixmap, gc,
-		   ball_pixbuf, 0, 0, 0, 0, boxsize * 4, boxsize * 7,
-		   GDK_RGB_DITHER_NORMAL, 0, 0);
+                   ball_pixbuf, 0, 0, 0, 0, boxsize * 4, boxsize * 7,
+                   GDK_RGB_DITHER_NORMAL, 0, 0);
 
   gdk_draw_rectangle (blank_pixmap, gc, TRUE, 0, 0, boxsize, boxsize);
 
@@ -337,26 +343,26 @@ refresh_preview_pixmaps (void)
 
   if (ball_preimage)
     scaled = games_preimage_render (ball_preimage, 4 * preview_width,
-				    7 * preview_height);
+                                    7 * preview_height);
 
   if (!scaled) {
     scaled = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8,
-			     4 * preview_width, 7 * preview_height);
+                             4 * preview_width, 7 * preview_height);
     gdk_pixbuf_fill (scaled, 0x00000000);
   }
 
 
   for (i = 0; i < 7; i++) {
     preview_pixmaps[i] = gdk_pixmap_new (widget->window,
-					 preview_width, preview_height, -1);
+                                         preview_width, preview_height, -1);
     gdk_draw_rectangle (preview_pixmaps[i],
-			widget->style->bg_gc[GTK_STATE_NORMAL],
-			TRUE, 0, 0, preview_width, preview_height);
+                        widget->style->bg_gc[GTK_STATE_NORMAL],
+                        TRUE, 0, 0, preview_width, preview_height);
 
     gdk_draw_pixbuf (preview_pixmaps[i], widget->style->white_gc,
-		     scaled, 0, i * preview_height, 0, 0,
-		     preview_width, preview_height,
-		     GDK_RGB_DITHER_NORMAL, 0, 0);
+                     scaled, 0, i * preview_height, 0, 0,
+                     preview_width, preview_height,
+                     GDK_RGB_DITHER_NORMAL, 0, 0);
   }
 
   if (blank_preview_pixmap)
@@ -364,8 +370,8 @@ refresh_preview_pixmaps (void)
   blank_preview_pixmap =
     gdk_pixmap_new (widget->window, preview_width, preview_height, -1);
   gdk_draw_rectangle (blank_preview_pixmap,
-		      widget->style->bg_gc[GTK_STATE_NORMAL], TRUE, 0, 0,
-		      preview_width, preview_height);
+                      widget->style->bg_gc[GTK_STATE_NORMAL], TRUE, 0, 0,
+                      preview_width, preview_height);
 
   g_object_unref (scaled);
 
@@ -377,17 +383,25 @@ draw_all_balls (GtkWidget * widget)
   gdk_window_invalidate_rect (widget->window, NULL, FALSE);
 }
 
-void
-set_inmove (int i)
+static void
+start_animation (void)
 {
   int ms;
 
+  ms = (inmove ? move_timeout : 100);
+  if (animate_id == 0)
+    animate_id = g_timeout_add (ms, animate, draw_area);
+}
+
+void
+set_inmove (int i)
+{
   if (inmove != i) {
     inmove = i;
-    ms = (inmove ? move_timeout : 100);
     if (animate_id)
       g_source_remove (animate_id);
-    animate_id = g_timeout_add (ms, animate, draw_area);
+    animate_id = 0;
+    start_animation ();
   }
 }
 
@@ -420,7 +434,7 @@ start_game (void)
   char string[20];
 
   set_statusbar_message (_
-			 ("Match five objects of the same type in a row to score!"));
+                         ("Match five objects of the same type in a row to score!"));
   refresh_screen ();
   active = -1;
   target = -1;
@@ -458,10 +472,10 @@ draw_preview (void)
 
     if (i < npieces)
       gdk_window_set_back_pixmap (preview_widgets[i]->window,
-				  preview_pixmaps[preview[i] - 1], FALSE);
+                                  preview_pixmaps[preview[i] - 1], FALSE);
     else
       gdk_window_set_back_pixmap (preview_widgets[i]->window,
-				  blank_preview_pixmap, FALSE);
+                                  blank_preview_pixmap, FALSE);
 
     gdk_window_clear (preview_widgets[i]->window);
   }
@@ -483,7 +497,7 @@ show_scores (gint pos, gboolean new_game)
   if (dialog == NULL) {
     dialog = games_scores_dialog_new (GTK_WINDOW (app), highscores, _("GNOME Five or More"));
     games_scores_dialog_set_category_description (GAMES_SCORES_DIALOG
-						  (dialog), _("_Board size:"));
+                                                  (dialog), _("_Board size:"));
   }
 
   if (pos > 0) {
@@ -495,15 +509,6 @@ show_scores (gint pos, gboolean new_game)
   gtk_widget_hide (dialog);
 }
 
-static void
-update_score_state ()
-{
-  GList *top;
-
-  top = games_scores_get (highscores);
-  gtk_widget_set_sensitive (scoreitem, top != NULL);
-}
-
 static void
 game_over (void)
 {
@@ -514,7 +519,6 @@ game_over (void)
   hiscore.plain = score;
   pos = games_scores_add_score (highscores, hiscore);
   show_scores (pos, TRUE);
-  update_score_state ();
   return;
 }
 
@@ -553,7 +557,7 @@ init_new_balls (int num, int prev)
     j = g_rand_int_range (rgen, 0, num_boxes);
     if (field[j].color == 0) {
       field[j].color = (prev == -1) ?
-	g_rand_int_range (rgen, 1, ncolors + 1) : preview[prev];
+        g_rand_int_range (rgen, 1, ncolors + 1) : preview[prev];
       i++;
     }
   }
@@ -564,7 +568,7 @@ void
 draw_box (GtkWidget * widget, int x, int y)
 {
   gtk_widget_queue_draw_area (widget, x * boxsize, y * boxsize,
-			      boxsize, boxsize);
+                              boxsize, boxsize);
 }
 
 static int
@@ -579,18 +583,18 @@ route (int num)
     if (field[i].pathsearch == num) {
       flag = 1;
       if ((i / hfieldsize > 0) && (field[i - hfieldsize].pathsearch == -1)
-	  && (field[i - hfieldsize].color == 0))
-	field[i - hfieldsize].pathsearch = num + 1;
+          && (field[i - hfieldsize].color == 0))
+        field[i - hfieldsize].pathsearch = num + 1;
       if ((i / hfieldsize < vfieldsize - 1)
-	  && (field[i + hfieldsize].pathsearch == -1)
-	  && (field[i + hfieldsize].color == 0))
-	field[i + hfieldsize].pathsearch = num + 1;
+          && (field[i + hfieldsize].pathsearch == -1)
+          && (field[i + hfieldsize].color == 0))
+        field[i + hfieldsize].pathsearch = num + 1;
       if ((i % hfieldsize > 0) && (field[i - 1].pathsearch == -1)
-	  && (field[i - 1].color == 0))
-	field[i - 1].pathsearch = num + 1;
+          && (field[i - 1].color == 0))
+        field[i - 1].pathsearch = num + 1;
       if ((i % hfieldsize < hfieldsize - 1) && (field[i + 1].pathsearch == -1)
-	  && (field[i + 1].color == 0)) {
-	field[i + 1].pathsearch = num + 1;
+          && (field[i + 1].color == 0)) {
+        field[i + 1].pathsearch = num + 1;
       }
     }
   }
@@ -664,14 +668,14 @@ cell_clicked (GtkWidget * widget, int fx, int fy)
 
       target = fx + fy * hfieldsize;
       if (find_route ()) {
-	/* We found a route to the new position */
+        /* We found a route to the new position */
 
-	set_inmove (1);
+        set_inmove (1);
       } else {
-	/* Can't move there! */
-	set_statusbar_message (_("You can't move there!"));
-	reset_pathsearch ();
-	target = -1;
+        /* Can't move there! */
+        set_statusbar_message (_("You can't move there!"));
+        reset_pathsearch ();
+        target = -1;
       }
     }
   } else {
@@ -685,15 +689,16 @@ cell_clicked (GtkWidget * widget, int fx, int fy)
       /* It's not active, so let's activate it! */
 
       if (active != -1) {
-	/* There is an other active ball, we should deactivate it first! */
+        /* There is an other active ball, we should deactivate it first! */
 
-	x = active % hfieldsize;
-	y = active / hfieldsize;
-	deactivate (widget, x, y);
+        x = active % hfieldsize;
+        y = active / hfieldsize;
+        deactivate (widget, x, y);
       }
 
       active = fx + fy * hfieldsize;
       field[active].active = 1;
+      start_animation();
     }
   }
 
@@ -870,18 +875,18 @@ field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
       idx = j + i * hfieldsize;
 
       if (field[idx].color != 0) {
-	phase = field[idx].phase;
-	color = field[idx].color - 1;
+        phase = field[idx].phase;
+        color = field[idx].color - 1;
 
-	phase = ABS (ABS (3 - phase) - 3);
+        phase = ABS (ABS (3 - phase) - 3);
 
-	gdk_draw_drawable (window, gc, ball_pixmap,
-			   phase * boxsize,
-			   color * boxsize,
-			   j * boxsize, i * boxsize, boxsize, boxsize);
+        gdk_draw_drawable (window, gc, ball_pixmap,
+                           phase * boxsize,
+                           color * boxsize,
+                           j * boxsize, i * boxsize, boxsize, boxsize);
       } else {
-	gdk_draw_drawable (window, gc, blank_pixmap,
-			   0, 0, j * boxsize, i * boxsize, boxsize, boxsize);
+        gdk_draw_drawable (window, gc, blank_pixmap,
+                           0, 0, j * boxsize, i * boxsize, boxsize, boxsize);
       }
     }
   }
@@ -892,7 +897,7 @@ field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
     GdkColor color;
 
     if (((backgnd.color.red + backgnd.color.green + backgnd.color.blue) / 3) >
-	(G_MAXUINT16 / 2))
+        (G_MAXUINT16 / 2))
       gdk_color_parse ("#000000", &color);
     else
       gdk_color_parse ("#FFFFFF", &color);
@@ -902,8 +907,8 @@ field_expose_event (GtkWidget * widget, GdkEventExpose * event, gpointer gp)
     gdk_gc_set_foreground (gc, &color);
 
     gdk_draw_rectangle (window, gc, FALSE,
-			cursor_x * boxsize + 1, cursor_y * boxsize + 1,
-			boxsize - 2, boxsize - 2);
+                        cursor_x * boxsize + 1, cursor_y * boxsize + 1,
+                        boxsize - 2, boxsize - 2);
   }
 
   g_object_unref (gc);
@@ -957,7 +962,7 @@ find_lines (int num)
 
   x++;
   while ((x <= hfieldsize - 1)
-	 && (field[x + y * hfieldsize].color == field[num].color)) {
+         && (field[x + y * hfieldsize].color == field[num].color)) {
     list[subcount] = x + y * hfieldsize;
     subcount++;
     x++;
@@ -980,7 +985,7 @@ find_lines (int num)
   x = num % hfieldsize;
   y++;
   while ((y <= vfieldsize - 1)
-	 && (field[x + y * hfieldsize].color == field[num].color)) {
+         && (field[x + y * hfieldsize].color == field[num].color)) {
     list[subcount] = x + y * hfieldsize;
     subcount++;
     y++;
@@ -1003,7 +1008,7 @@ find_lines (int num)
   x = num % hfieldsize + 1;
   y = num / hfieldsize + 1;
   while ((y <= vfieldsize - 1) && (x <= hfieldsize - 1)
-	 && (field[x + y * hfieldsize].color == field[num].color)) {
+         && (field[x + y * hfieldsize].color == field[num].color)) {
     list[subcount] = x + y * hfieldsize;
     subcount++;
     y++;
@@ -1012,7 +1017,7 @@ find_lines (int num)
   x = num % hfieldsize - 1;
   y = num / hfieldsize - 1;
   while ((y >= 0) && (x >= 0)
-	 && (field[x + y * hfieldsize].color == field[num].color)) {
+         && (field[x + y * hfieldsize].color == field[num].color)) {
     list[subcount] = x + y * hfieldsize;
     subcount++;
     y--;
@@ -1030,7 +1035,7 @@ find_lines (int num)
   x = num % hfieldsize + 1;
   y = num / hfieldsize - 1;
   while ((y >= 0) && (x <= hfieldsize - 1)
-	 && (field[x + y * hfieldsize].color == field[num].color)) {
+         && (field[x + y * hfieldsize].color == field[num].color)) {
     list[subcount] = x + y * hfieldsize;
     subcount++;
     y--;
@@ -1039,7 +1044,7 @@ find_lines (int num)
   x = num % hfieldsize - 1;
   y = num / hfieldsize + 1;
   while ((y <= vfieldsize - 1) && (x >= 0)
-	 && (field[x + y * hfieldsize].color == field[num].color)) {
+         && (field[x + y * hfieldsize].color == field[num].color)) {
     list[subcount] = x + y * hfieldsize;
     subcount++;
     y++;
@@ -1108,27 +1113,30 @@ animate (gpointer gp)
   GtkWidget *widget = GTK_WIDGET (gp);
   int x, y;
   int newactive = 0;
-
+    
   x = active % hfieldsize;
   y = active / hfieldsize;
 
-  if (active == -1)
-    return TRUE;
+  if (active == -1) {
+    animate_id = 0;
+    return FALSE;
+  }
+
   if (inmove != 0) {
     if ((x > 0)
-	&& (field[active - 1].pathsearch == field[active].pathsearch + 1))
+        && (field[active - 1].pathsearch == field[active].pathsearch + 1))
       newactive = active - 1;
     else if ((x < hfieldsize - 1)
-	     && (field[active + 1].pathsearch ==
-		 field[active].pathsearch + 1))
+             && (field[active + 1].pathsearch ==
+                 field[active].pathsearch + 1))
       newactive = active + 1;
     else if ((y > 0)
-	     && (field[active - hfieldsize].pathsearch ==
-		 field[active].pathsearch + 1))
+             && (field[active - hfieldsize].pathsearch ==
+                 field[active].pathsearch + 1))
       newactive = active - hfieldsize;
     else if ((y < vfieldsize - 1)
-	     && (field[active + hfieldsize].pathsearch ==
-		 field[active].pathsearch + 1))
+             && (field[active + hfieldsize].pathsearch ==
+                 field[active].pathsearch + 1))
       newactive = active + hfieldsize;
     else {
       set_inmove (0);
@@ -1150,27 +1158,27 @@ animate (gpointer gp)
       draw_box (widget, x, y);
       reset_pathsearch ();
       if (!check_goal (widget, newactive)) {
-	gboolean fullline;
-	int balls[npieces];
-	int spaces;
+        gboolean fullline;
+        int balls[npieces];
+        int spaces;
 
-	clear_tags ();
-	fullline = FALSE;
-	spaces = spaces_left ();
-	if (spaces > npieces)
-	  spaces = npieces;
-	for (x = 0; x < spaces; x++) {
-	  int tmp = init_new_balls (1, x);
-	  draw_box (widget, tmp % hfieldsize, tmp / hfieldsize);
-	  balls[x] = tmp;
-	  fullline = (find_lines (balls[x]) >= 5) || fullline;
-	}
-	if (fullline)
-	  addscore (kill_tags (widget));
-	if (check_gameover () == -1)
-	  return FALSE;
-	init_preview ();
-	draw_preview ();
+        clear_tags ();
+        fullline = FALSE;
+        spaces = spaces_left ();
+        if (spaces > npieces)
+          spaces = npieces;
+        for (x = 0; x < spaces; x++) {
+          int tmp = init_new_balls (1, x);
+          draw_box (widget, tmp % hfieldsize, tmp / hfieldsize);
+          balls[x] = tmp;
+          fullline = (find_lines (balls[x]) >= 5) || fullline;
+        }
+        if (fullline)
+          addscore (kill_tags (widget));
+        if (check_gameover () == -1)
+          return FALSE;
+        init_preview ();
+        draw_preview ();
       }
       return TRUE;
     }
@@ -1211,18 +1219,18 @@ game_about_callback (GtkAction * action, gpointer * data)
 #else
                          "name", _("Five or More"),
 #endif
-			 "version", VERSION,
-			 "comments", _("GNOME port of the once-popular Color Lines game.\n\nFive or More is a part of GNOME Games."),
-			 "copyright",
-			 "Copyright \xc2\xa9 1997-2008 Free Software Foundation, Inc.",
-			 "license", license,
+                         "version", VERSION,
+                         "comments", _("GNOME port of the once-popular Color Lines game.\n\nFive or More is a part of GNOME Games."),
+                         "copyright",
+                         "Copyright \xc2\xa9 1997-2008 Free Software Foundation, Inc.",
+                         "license", license,
                          "authors", authors,
-			 "documenters", documenters,
-			 "translator-credits", _("translator-credits"),
-			 "logo-icon-name", "gnome-glines",
-			 "website", "http://www.gnome.org/projects/gnome-games/",
+                         "documenters", documenters,
+                         "translator-credits", _("translator-credits"),
+                         "logo-icon-name", "gnome-glines",
+                         "website", "http://www.gnome.org/projects/gnome-games/",
                          "website-label", _("GNOME Games web site"),
-			 "wrap-license", TRUE, NULL);
+                         "wrap-license", TRUE, NULL);
   g_free (license);
 }
 
@@ -1365,23 +1373,26 @@ set_selection (GtkWidget * widget, char *data)
   const gchar *entry;
 
   entry = games_file_list_get_nth (theme_file_list,
-				   gtk_combo_box_get_active (GTK_COMBO_BOX
-							     (widget)));
+                                   gtk_combo_box_get_active (GTK_COMBO_BOX
+                                                             (widget)));
   games_conf_set_string (KEY_PREFERENCES_GROUP, KEY_BALL_THEME, entry);
 }
 
 static GtkWidget *
 fill_menu (void)
 {
+  const char *pixmap_dir;
+
   if (theme_file_list)
     g_object_unref (theme_file_list);
 
-  theme_file_list = games_file_list_new_images (PIXMAPDIR, NULL);
+  pixmap_dir = games_runtime_get_directory (GAMES_RUNTIME_GAME_PIXMAP_DIRECTORY);
+  theme_file_list = games_file_list_new_images (pixmap_dir, NULL);
   games_file_list_transform_basename (theme_file_list);
 
   return games_file_list_create_widget (theme_file_list, ball_filename,
-					GAMES_FILE_LIST_REMOVE_EXTENSION |
-					GAMES_FILE_LIST_REPLACE_UNDERSCORES);
+                                        GAMES_FILE_LIST_REMOVE_EXTENSION |
+                                        GAMES_FILE_LIST_REPLACE_UNDERSCORES);
 }
 
 static void
@@ -1438,15 +1449,15 @@ game_props_callback (void)
 
   if (!pref_dialog) {
     pref_dialog = gtk_dialog_new_with_buttons (_("Five or More Preferences"),
-					       GTK_WINDOW (app),
-					       GTK_DIALOG_DESTROY_WITH_PARENT,
-					       GTK_STOCK_CLOSE,
-					       GTK_RESPONSE_CLOSE, NULL);
+                                               GTK_WINDOW (app),
+                                               GTK_DIALOG_DESTROY_WITH_PARENT,
+                                               GTK_STOCK_CLOSE,
+                                               GTK_RESPONSE_CLOSE, NULL);
     gtk_dialog_set_has_separator (GTK_DIALOG (pref_dialog), FALSE);
     g_signal_connect (pref_dialog, "response",
-		      G_CALLBACK (pref_dialog_response), NULL);
+                      G_CALLBACK (pref_dialog_response), NULL);
     g_signal_connect (pref_dialog, "delete-event",
-		      G_CALLBACK (gtk_widget_hide), NULL);
+                      G_CALLBACK (gtk_widget_hide), NULL);
 
     gtk_window_set_resizable (GTK_WINDOW (pref_dialog), FALSE);
     gtk_container_set_border_width (GTK_CONTAINER (pref_dialog), 5);
@@ -1455,7 +1466,7 @@ game_props_callback (void)
     vbox = gtk_vbox_new (FALSE, 18);
     gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
     gtk_box_pack_start (GTK_BOX (GTK_DIALOG (pref_dialog)->vbox), vbox,
-			FALSE, FALSE, 0);
+                        FALSE, FALSE, 0);
 
     frame = games_frame_new (_("Themes"));
     table = gtk_table_new (2, 2, FALSE);
@@ -1471,7 +1482,7 @@ game_props_callback (void)
 
     omenu = fill_menu ();
     g_signal_connect (omenu, "changed",
-		      G_CALLBACK (set_selection), NULL);
+                      G_CALLBACK (set_selection), NULL);
     gtk_table_attach_defaults (GTK_TABLE (table), omenu, 1, 2, 0, 1);
     gtk_label_set_mnemonic_widget (GTK_LABEL (l), omenu);
 
@@ -1484,7 +1495,7 @@ game_props_callback (void)
       w = gtk_color_button_new ();
       gtk_color_button_set_color (GTK_COLOR_BUTTON (w), &backgnd.color);
       g_signal_connect (w, "color-set",
-			G_CALLBACK (bg_color_callback), NULL);
+                        G_CALLBACK (bg_color_callback), NULL);
     }
 
     gtk_table_attach_defaults (GTK_TABLE (table), w, 1, 2, 1, 2);
@@ -1501,7 +1512,7 @@ game_props_callback (void)
     if (game_size == SMALL)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
     g_signal_connect (button, "clicked",
-		      G_CALLBACK (size_callback), (gpointer) SMALL);
+                      G_CALLBACK (size_callback), (gpointer) SMALL);
 
     gtk_container_add (GTK_CONTAINER (fv), button);
 
@@ -1511,7 +1522,7 @@ game_props_callback (void)
     if (game_size == MEDIUM)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
     g_signal_connect (button, "clicked",
-		      G_CALLBACK (size_callback), (gpointer) MEDIUM);
+                      G_CALLBACK (size_callback), (gpointer) MEDIUM);
     gtk_container_add (GTK_CONTAINER (fv), button);
 
     button = gtk_radio_button_new_with_mnemonic
@@ -1519,7 +1530,7 @@ game_props_callback (void)
     if (game_size == LARGE)
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), TRUE);
     g_signal_connect (button, "clicked",
-		      G_CALLBACK (size_callback), (gpointer) LARGE);
+                      G_CALLBACK (size_callback), (gpointer) LARGE);
     gtk_container_add (GTK_CONTAINER (fv), button);
 
 
@@ -1533,10 +1544,10 @@ game_props_callback (void)
       gtk_check_button_new_with_mnemonic (_("_Use fast moves"));
     if (move_timeout == 10) {
       gtk_toggle_button_set_active
-	(GTK_TOGGLE_BUTTON (fast_moves_toggle_button), TRUE);
+        (GTK_TOGGLE_BUTTON (fast_moves_toggle_button), TRUE);
     }
     g_signal_connect (fast_moves_toggle_button, "clicked",
-		      G_CALLBACK (set_fast_moves_callback), NULL);
+                      G_CALLBACK (set_fast_moves_callback), NULL);
 
     gtk_container_add (GTK_CONTAINER (fv), fast_moves_toggle_button);
 
@@ -1560,11 +1571,7 @@ game_quit_callback (GtkAction * action, gpointer data)
 static void
 game_help_callback (GtkAction * action, gpointer data)
 {
-#ifdef HAVE_GNOME
-  gnome_help_display ("glines.xml", NULL, NULL);
-#else
-#warning need to port help to gtk-only!
-#endif /* HAVE_GNOME */
+  games_help_display (app, "glines", NULL);
 }
 
 static int
@@ -1591,7 +1598,7 @@ configure_event_callback (GtkWidget * widget, GdkEventConfigure * event)
 
   boxsize = (event->width - 1) / hfieldsize;
   ball_pixmap = gdk_pixmap_new (draw_area->window, boxsize * 4, boxsize * 7,
-				-1);
+                                -1);
   blank_pixmap = gdk_pixmap_new (draw_area->window, boxsize, boxsize, -1);
   refresh_pixmaps ();
 
@@ -1600,17 +1607,15 @@ configure_event_callback (GtkWidget * widget, GdkEventConfigure * event)
   return TRUE;
 }
 
-#ifdef HAVE_GNOME
-
+#ifdef WITH_SMCLIENT
 static int
-save_state (GnomeClient * client,
-	    gint phase,
-	    GnomeSaveStyle save_style,
-	    gint shutdown,
-	    GnomeInteractStyle interact_style,
-	    gint fast, gpointer client_data)
+save_state_cb (EggSMClient *client,
+            GKeyFile* keyfile,
+            gpointer client_data)
 {
   gchar *buf;
+  int argc = 0;
+  char *argv[1];
   int i;
 
   games_conf_set_integer (KEY_SAVED_GROUP, KEY_SAVED_SCORE, score);
@@ -1630,18 +1635,23 @@ save_state (GnomeClient * client,
   games_conf_set_string (KEY_SAVED_GROUP, KEY_SAVED_PREVIEW, buf);
   g_free (buf);
 
+  argv[argc++] = g_get_prgname ();
+
+  egg_sm_client_set_restart_command (client, argc, (const char **) argv);
+
   return TRUE;
 }
 
 static gint
-client_die (GnomeClient * client, gpointer client_data)
+quit_cb (EggSMClient *client,
+         gpointer client_data)
 {
   gtk_main_quit ();
 
   return FALSE;
 }
 
-#endif /* HAVE_GNOME */
+#endif /* WITH_SMCLIENT */
 
 static void
 load_properties (void)
@@ -1680,7 +1690,7 @@ restart (void)
     for (i = 0; i < hfieldsize * vfieldsize; i++) {
       field[i].color = CLAMP (buf[i * 4] - 'h', 1, ncolors);
       field[i].pathsearch =
-	CLAMP (buf[i * 4 + 1] - 'h', -1, hfieldsize * vfieldsize);
+        CLAMP (buf[i * 4 + 1] - 'h', -1, hfieldsize * vfieldsize);
       field[i].phase = CLAMP (buf[i * 4 + 2] - 'h', 0, 3);
       field[i].active = CLAMP (buf[i * 4 + 3] - 'h', -1, 1);
     }
@@ -1743,7 +1753,7 @@ create_menus (GtkUIManager * ui_manager)
   action_group = gtk_action_group_new ("MenuActions");
   gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
   gtk_action_group_add_actions (action_group, actions,
-				G_N_ELEMENTS (actions), NULL);
+                                G_N_ELEMENTS (actions), NULL);
 
   fullscreen_action =
     gtk_action_group_get_action (action_group, "Fullscreen");
@@ -1757,20 +1767,10 @@ create_menus (GtkUIManager * ui_manager)
   gtk_window_add_accel_group (GTK_WINDOW (app), accel_group);
 
   scoreitem = gtk_ui_manager_get_widget (ui_manager,
-					 "/MainMenu/GameMenu/Scores");
+                                         "/MainMenu/GameMenu/Scores");
   menubar = gtk_ui_manager_get_widget (ui_manager, "/MainMenu");
 }
 
-#ifdef HAVE_GNOME
-#ifndef GNOME_CLIENT_RESTARTED
-#define GNOME_CLIENT_RESTARTED(client) \
-(GNOME_CLIENT_CONNECTED (client) && \
- (gnome_client_get_previous_id (client) != NULL) && \
- (strcmp (gnome_client_get_id (client), \
-  gnome_client_get_previous_id (client)) == 0))
-#endif /* GNOME_CLIENT_RESTARTED */
-#endif /* HAVE_GNOME */
-
 static void
 init_config (void)
 {
@@ -1795,13 +1795,11 @@ main (int argc, char *argv[])
   GtkWidget *preview_hbox;
   GtkUIManager *ui_manager;
   guint i;
-#ifdef HAVE_GNOME
-  GnomeClient *client;
-  GnomeProgram *program;
-#else
   gboolean retval;
   GError *error = NULL;
-#endif
+#ifdef WITH_SMCLIENT
+  EggSMClient *sm_client;
+#endif /* WITH_SMCLIENT */
 
 #if defined(HAVE_GNOME) || defined(HAVE_RSVG_GNOMEVFS)
   /* If we're going to use gnome-vfs, we need to init threads before
@@ -1810,11 +1808,14 @@ main (int argc, char *argv[])
   g_thread_init (NULL);
 #endif
 
+  if (!games_runtime_init ("glines"))
+    return 1;
+
   setgid_io_init ();
 
   rgen = g_rand_new ();
 
-  bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
+  bindtextdomain (GETTEXT_PACKAGE, games_runtime_get_directory (GAMES_RUNTIME_LOCALE_DIRECTORY));
   bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
   textdomain (GETTEXT_PACKAGE);
 
@@ -1823,15 +1824,10 @@ main (int argc, char *argv[])
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
 #endif
 
-#ifdef HAVE_GNOME
-  program = gnome_program_init ("glines", VERSION,
-				LIBGNOMEUI_MODULE,
-				argc, argv,
-				GNOME_PARAM_GOPTION_CONTEXT, context,
-				GNOME_PARAM_APP_DATADIR, DATADIR, /* FIXMEchpe: this ought to use SHAREDIR !! */
-                                NULL);
-#else
   g_option_context_add_group (context, gtk_get_option_group (TRUE));
+#ifdef WITH_SMCLIENT
+  g_option_context_add_group (context, egg_sm_client_get_option_group ());
+#endif /* WITH_SMCLIENT */
 
   retval = g_option_context_parse (context, &argc, &argv, &error);
   g_option_context_free (context);
@@ -1840,7 +1836,8 @@ main (int argc, char *argv[])
     g_error_free (error);
     exit (1);
   }
-#endif /* HAVE_GNOME */
+
+  g_set_application_name (_("Five or More"));
 
   games_conf_initialise ("GLines");
 
@@ -1852,32 +1849,30 @@ main (int argc, char *argv[])
 
   gtk_window_set_default_icon_name ("gnome-glines");
 
-#ifdef HAVE_GNOME
-  client = gnome_master_client ();
-  g_signal_connect (client, "save-yourself",
-		    G_CALLBACK (save_state), argv[0]);
-  g_signal_connect (client, "die",
-                    G_CALLBACK (client_die), NULL);
-
-  if (GNOME_CLIENT_RESTARTED (client))
+#ifdef WITH_SMCLIENT
+  sm_client = egg_sm_client_get ();
+  g_signal_connect (sm_client, "save-state",
+                    G_CALLBACK (save_state_cb), NULL);
+  g_signal_connect (sm_client, "quit",
+                    G_CALLBACK (quit_cb), NULL);
+  if (egg_sm_client_is_resumed(sm_client))
     restart ();
   else
     reset_game ();
 #else
   reset_game ();
-#endif /* HAVE_GNOME */
+#endif /* WITH_SMCLIENT */
 
   app = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 
   gtk_window_set_title (GTK_WINDOW (app), _("Five or More"));
 
   gtk_window_set_default_size (GTK_WINDOW (app), DEFAULT_WIDTH, DEFAULT_HEIGHT);
-  games_conf_add_window (GTK_WINDOW (app), NULL);
-
   g_signal_connect (app, "delete-event",
-		    G_CALLBACK (game_quit_callback), NULL);
+                    G_CALLBACK (game_quit_callback), NULL);
   g_signal_connect (app, "window-state-event",
-		    G_CALLBACK (window_state_callback), NULL);
+                    G_CALLBACK (window_state_callback), NULL);
+  games_conf_add_window (GTK_WINDOW (app), NULL);
 
   statusbar = gtk_statusbar_new ();
   ui_manager = gtk_ui_manager_new ();
@@ -1915,14 +1910,14 @@ main (int argc, char *argv[])
   for (i = 0; i < MAXNPIECES; i++) {
     preview_widgets[i] = gtk_drawing_area_new ();
     gtk_box_pack_start (GTK_BOX (preview_hbox),
-			preview_widgets[i], TRUE, TRUE, 0);
+                        preview_widgets[i], TRUE, TRUE, 0);
     /* So we have all the windows at configure time since
      * we only hook into one of the configure events. */
     /* Yes, this is an evil hack. */
     gtk_widget_realize (preview_widgets[i]);
   }
   g_signal_connect (preview_widgets[0], "configure-event",
-		    G_CALLBACK (preview_configure_cb), NULL);
+                    G_CALLBACK (preview_configure_cb), NULL);
 
   scorelabel = gtk_label_new (NULL);
 
@@ -1937,13 +1932,13 @@ main (int argc, char *argv[])
 
   draw_area = gtk_drawing_area_new ();
   g_signal_connect (draw_area, "button-press-event",
-		    G_CALLBACK (button_press_event), NULL);
+                    G_CALLBACK (button_press_event), NULL);
   g_signal_connect (draw_area, "key-press-event",
-		    G_CALLBACK (key_press_event), NULL);
+                    G_CALLBACK (key_press_event), NULL);
   g_signal_connect (draw_area, "configure-event",
-		    G_CALLBACK (configure_event_callback), NULL);
+                    G_CALLBACK (configure_event_callback), NULL);
   g_signal_connect (draw_area, "expose-event",
-		    G_CALLBACK (field_expose_event), NULL);
+                    G_CALLBACK (field_expose_event), NULL);
   gridframe = games_grid_frame_new (hfieldsize, vfieldsize);
   games_grid_frame_set_padding (GAMES_GRID_FRAME (gridframe), 1, 1);
   gtk_container_add (GTK_CONTAINER (gridframe), draw_area);
@@ -1953,14 +1948,12 @@ main (int argc, char *argv[])
   relay_table ();
 
   gtk_widget_set_events (draw_area,
-			 gtk_widget_get_events (draw_area) |
-			 GDK_BUTTON_PRESS_MASK | GDK_KEY_PRESS_MASK);
+                         gtk_widget_get_events (draw_area) |
+                         GDK_BUTTON_PRESS_MASK | GDK_KEY_PRESS_MASK);
 
   GTK_WIDGET_SET_FLAGS (draw_area, GTK_CAN_FOCUS);
   gtk_widget_grab_focus (draw_area);
 
-  update_score_state ();
-
   load_properties ();
 
   gtk_widget_show_all (app);
@@ -1975,9 +1968,12 @@ main (int argc, char *argv[])
 
   games_conf_shutdown ();
 
-#ifdef HAVE_GNOME
-  g_object_unref (program);
-#endif /* HAVE_GNOME */
+#ifdef WITH_SMCLIENT
+  g_signal_handlers_disconnect_matched (sm_client, G_SIGNAL_MATCH_DATA,
+                                        0, 0, NULL, NULL, NULL);
+#endif /* WITH_SMCLIENT */
+
+  games_runtime_shutdown ();
 
   return 0;
 }
