@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 try:
     import pygtk
     pygtk.require('2.0')
@@ -21,7 +22,11 @@ from dialog_swallower import SwappableArea
 icon_factory = gtk.IconFactory()
 STOCK_PIXBUFS = {}
 for filename,stock_id in [('footprints.png','tracks'),]:
-    pb = gtk.gdk.pixbuf_new_from_file(os.path.join(IMAGE_DIR,filename))
+    try:
+        pb = gtk.gdk.pixbuf_new_from_file(os.path.join(IMAGE_DIR,filename))
+    except gobject.GError, e:
+        print 'Failed to load pixbuf: %s' % e
+        continue
     STOCK_PIXBUFS[stock_id]=pb
     iconset = gtk.IconSet(pb)
     icon_factory.add(stock_id,iconset)
@@ -221,6 +226,8 @@ class UI (gconf_wrapper.GConfWrapper):
             self.tb.show()
         if self.gconf['always_show_hints']:
             self.gsd.update_all_hints()
+        if self.gconf['highlight']:
+            self.gsd.toggle_highlight(True)
 
 
     def show (self):
