@@ -136,7 +136,6 @@ static GtkWidget *dialog = NULL;
 static gint modify_dice (GtkWidget * widget, gpointer data);
 static gint roll_dice (GtkWidget * widget, GdkEvent * event,
 			     gpointer data);
-void update_score_state (void);
 static void UpdateRollLabel (void);
 
 static void
@@ -200,13 +199,15 @@ CheerWinner (void)
     pos = games_scores_add_score (highscores, score);
 
     if (pos > 0) {
+      games_scores_update_score (highscores, players[winner].name);
       if (dialog) {
         gtk_window_present (GTK_WINDOW (dialog));
       } else {
         dialog = games_scores_dialog_new (GTK_WINDOW (window), highscores, _("Tali Scores"));
         message =
   	  g_strdup_printf ("<b>%s</b>\n\n%s", _("Congratulations!"),
-			 _("Your score has made the top ten."));
+                           pos == 1 ? _("Your score is the best!") :
+                           _("Your score has made the top ten."));
         games_scores_dialog_set_message (GAMES_SCORES_DIALOG (dialog), message);
         g_free (message);
       }
@@ -215,8 +216,6 @@ CheerWinner (void)
       gtk_dialog_run (GTK_DIALOG (dialog));
       gtk_widget_hide (dialog);
     }
-
-    update_score_state ();
   }
 
   if (players[winner].name)
@@ -602,6 +601,9 @@ about_cb (GtkAction * action, gpointer data)
     "",
     N_("Console version (1992):"),
     "Orest Zborowski",
+    "",
+    N_("Colors game and multi-level AI (2006):"),
+    "Geoff Buchan",
     NULL
   };
 
@@ -687,15 +689,6 @@ LoadDicePixmaps (void)
   }
 }
 
-void
-update_score_state (void)
-{
-  GList *top;
-
-  top = games_scores_get (highscores);
-  gtk_action_set_sensitive (scores_action, top != NULL);
-}
-
 void
 update_undo_sensitivity (void)
 {
@@ -871,9 +864,6 @@ GyahtzeeCreateMainWindow (void)
   gtk_widget_show (vbox);
 
   gtk_widget_show (window);
-
-  update_score_state ();
-
 }
 
 int
