@@ -25,7 +25,6 @@
 #include <errno.h>
 
 #include <glib.h>
-#include <glib/gthread.h>
 
 #include <gtk/gtk.h>
 
@@ -38,7 +37,7 @@ main (int argc, char *argv[])
   char *basepath = NULL, *kfname, *kfpath;
   GamesCardTheme *theme = NULL;
   GKeyFile *key_file = NULL;
-  guint i;
+  int i;
   int retval = EXIT_FAILURE;
   int *sizes = NULL, n_sizes = 0;
   char *data = NULL;
