@@ -60,59 +60,49 @@
 #define DEFAULT_TILESET "default.png"
 #define DEFAULT_MAPSET "Easy"
 
-tilepos *pos = 0;
+static GtkWidget *window, *statusbar;
+static GtkWidget *tiles_label;
+static GtkWidget *toolbar;
+static GtkWidget *moves_label, *chrono;
+static GtkWidget *warn_cb = NULL, *confirm_cb = NULL;
+static GtkWidget *colour_well = NULL;
+static GtkWidget *pref_dialog = NULL;
 
-tile tiles[MAX_TILES];
+static GtkAction *pause_action;
+static GtkAction *resume_action;
+static GtkAction *hint_action;
+static GtkAction *redo_action;
+static GtkAction *undo_action;
+static GtkAction *restart_action;
+static GtkAction *scores_action;
+static GtkAction *show_toolbar_action;
+static GtkAction *fullscreen_action;
+static GtkAction *leavefullscreen_action;
 
-GtkWidget *window, *statusbar;
-GtkWidget *tiles_label;
-GtkWidget *toolbar;
-
-GtkAction *pause_action;
-GtkAction *resume_action;
-GtkAction *hint_action;
-GtkAction *redo_action;
-GtkAction *undo_action;
-GtkAction *restart_action;
-GtkAction *scores_action;
-GtkAction *show_toolbar_action;
-GtkAction *fullscreen_action;
-GtkAction *leavefullscreen_action;
-
-gint selected_tile, visible_tiles;
-gint sequence_number;
-
-GList *tileset_list = NULL;
+/* Available tilsets */
+static GList *tileset_list = NULL;
 
 gchar *tileset = NULL;
-static gint mapset = -1;
-static gchar *score_current_mapset = NULL;
 
+tile tiles[MAX_TILES];
+static gint visible_tiles; /* Count of how many tiles are active */
+
+tilepos *pos = 0;
+static gint mapset = -1, active_mapset = -1;
 static gchar *selected_tileset = NULL;
+static gint selected_tile;
+static gint sequence_number;
+static gboolean undo_state = FALSE;
+static gboolean redo_state = FALSE;
+static gint moves_left = 0; /* The number of moves that can be made */
+gboolean paused = FALSE; /* true if the game is paused */
 
-gboolean undo_state = FALSE;
-gboolean redo_state = FALSE;
+/* Hint animation */
+static gint hint_tiles[2];
+static guint timer = 0;
+static guint timeout_counter = HINT_BLINK_NUM + 1;
 
-gint hint_tiles[2];
-guint timer;
-guint timeout_counter = HINT_BLINK_NUM + 1;
-
-GtkWidget *moves_label;
-gint moves_left = 0;
-GtkWidget *chrono;
-gint paused = 0;
-
-/* for the preferences */
-GtkWidget *warn_cb = NULL, *confirm_cb = NULL;
-GtkWidget *colour_well = NULL;
-GtkWidget *pref_dialog = NULL;
-
-/* Has the map been changed ? */
-gboolean new_map = TRUE;
-
-GamesScores *highscores;
-
-static gint get_mapset_index (void);
+static GamesScores *highscores;
 
 static const GamesScoresDescription scoredesc = { NULL,
   "Easy",
@@ -128,16 +118,15 @@ enum {
   GAME_DEAD
 } game_over = GAME_WAITING;
 
+static gint get_mapset_index (void);
 static void clear_undo_queue (void);
 void you_won (void);
 void check_free (void);
 void undo_tile_callback (void);
 void properties_callback (void);
 void shuffle_tiles_callback (void);
-void ensure_pause_off (void);
 void pause_callback (void);
-void new_game (void);
-void restart_game (void);
+void new_game (gboolean shuffle);
 
 void
 mahjongg_theme_warning (gchar * message)
@@ -310,11 +299,9 @@ conf_value_changed_cb (GamesConf *conf,
 
     mapset = get_mapset_index ();
 
-    new_map = TRUE;
-
     /* Skip the dialog if a game isn't in play. */
     if (game_over || !games_clock_get_seconds (GAMES_CLOCK (chrono))) {
-      new_game ();
+      new_game (TRUE);
       return;
     }
 
@@ -322,17 +309,17 @@ conf_value_changed_cb (GamesConf *conf,
                                                  GTK_DIALOG_MODAL,
                                                  GTK_MESSAGE_QUESTION,
                                                  GTK_BUTTONS_NONE,
-                                                _("Do you want to finish the current game or start playing with the new map immediately?"));
+                                                _("Do you want to start a new game with this map?"));
     gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-                                               _("If you choose to finish with the old map then the next game will use the new map."));
+                                               _("If you continue playing the next game will use the new map."));
     gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                            _("_Finish"), GTK_RESPONSE_REJECT,
-                            GTK_STOCK_NEW, GTK_RESPONSE_ACCEPT,
+                            _("_Continue playing"), GTK_RESPONSE_REJECT,
+                            _("Use _new map"), GTK_RESPONSE_ACCEPT,
                             NULL);
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT);
     response = gtk_dialog_run (GTK_DIALOG (dialog));
     if (response == GTK_RESPONSE_ACCEPT)
-      new_game ();
+      new_game (TRUE);
     gtk_widget_destroy (dialog);
   }
 }
@@ -435,7 +422,7 @@ message_flash (gchar * message)
     gtk_statusbar_get_context_id (GTK_STATUSBAR (statusbar), "flash");
   flashid =
     gtk_statusbar_push (GTK_STATUSBAR (statusbar), context_id, message);
-  g_timeout_add (5000, (GSourceFunc) message_flash_remove,
+  g_timeout_add_seconds (5, (GSourceFunc) message_flash_remove,
 		 GUINT_TO_POINTER (flashid));
 }
 
@@ -472,7 +459,7 @@ static void
 remove_pair (gint tile1, gint tile2)
 {
   gchar *tmpstr;
-
+   
   tiles[tile1].visible = tiles[tile2].visible = 0;
   tiles[tile1].selected &= ~SELECTED_FLAG;
   tiles[tile2].selected &= ~SELECTED_FLAG;
@@ -486,14 +473,13 @@ remove_pair (gint tile1, gint tile2)
   tmpstr = g_strdup_printf ("%3d", visible_tiles);
   gtk_label_set_text (GTK_LABEL (tiles_label), tmpstr);
   g_free (tmpstr);
-  update_moves_left ();
   set_undoredo_state (TRUE, FALSE);
 
+  update_moves_left ();   
   if (visible_tiles <= 0) {
     games_clock_stop (GAMES_CLOCK (chrono));
     you_won ();
   }
-
 }
 
 void
@@ -622,6 +608,8 @@ check_free (void)
 
     update_menu_sensitivities ();
     if (!game_over) {
+      gint response_id;
+       
       mb = gtk_message_dialog_new (GTK_WINDOW (window),
 				   GTK_DIALOG_MODAL
 				   | GTK_DIALOG_DESTROY_WITH_PARENT,
@@ -629,14 +617,27 @@ check_free (void)
 				   GTK_BUTTONS_NONE,
 				   (_("There are no more moves.")));
       gtk_dialog_add_buttons (GTK_DIALOG (mb),
-			      GTK_STOCK_UNDO,
-			      GTK_RESPONSE_REJECT,
-			      _("Shuffle"), GTK_RESPONSE_ACCEPT, NULL);
+			      GTK_STOCK_UNDO, GTK_RESPONSE_REJECT,
+                              _("_New game"), GTK_RESPONSE_CANCEL, NULL);
+
+      /* Can only shuffle if two tiles are visible to be matched. This cannot occur
+       * if all tiles are in a stack */
+      if (visible_tiles >= 2)
+        gtk_dialog_add_button(GTK_DIALOG (mb), _("_Shuffle"), GTK_RESPONSE_ACCEPT);
+
       gtk_dialog_set_default_response (GTK_DIALOG (mb), GTK_RESPONSE_ACCEPT);
-      if (gtk_dialog_run (GTK_DIALOG (mb)) == GTK_RESPONSE_ACCEPT)
-	shuffle_tiles_callback ();
-      else
-	undo_tile_callback ();
+      response_id = gtk_dialog_run (GTK_DIALOG (mb));
+      switch (response_id) {
+      case GTK_RESPONSE_ACCEPT:
+	 shuffle_tiles_callback ();
+         break;
+      case GTK_RESPONSE_CANCEL:
+         new_game (TRUE);
+         break;
+      default:
+	 undo_tile_callback ();
+         break;
+      }       
 
       gtk_widget_destroy (mb);
     }
@@ -669,6 +670,7 @@ you_won (void)
 						    (dialog), _("Map:"));
       message =
 	g_strdup_printf ("<b>%s</b>\n\n%s", _("Congratulations!"),
+                         pos == 1 ? _("Your score is the best!") :
 			 _("Your score has made the top ten."));
       games_scores_dialog_set_message (GAMES_SCORES_DIALOG (dialog), message);
       g_free (message);
@@ -831,7 +833,8 @@ stop_hints (void)
 static void
 hint_callback (void)
 {
-  gint i, j, free = 0, type;
+  gint i, j, type;
+  gboolean have_match = FALSE;
 
   if (paused || (game_over != GAME_RUNNING && game_over != GAME_WAITING))
     return;
@@ -845,29 +848,47 @@ hint_callback (void)
    * Tile Free is now _so_ much quicker, it is more elegant to do a
    * British Library search, and safer. */
   /* Note: British Library should probably read British Museum.  */
-
-  /* Clear any selection */
+   
+  /* Check if the selected tile has a match */
   if (selected_tile < MAX_TILES) {
-    tiles[selected_tile].selected = 0;
+    type = tiles[selected_tile].type;
+    for (i = 0; i < MAX_TILES && !have_match; i++) {
+       if (tiles[i].type == type && i != selected_tile && tile_free (i)) {
+         have_match = TRUE;
+         hint_tiles[0] = selected_tile;
+         hint_tiles[1] = i;
+       }
+    }
+  }
+
+  /* Check if any tiles match */
+  for (i = 0; i < MAX_TILES && !have_match; i++) {
+    if (tile_free (i)) {
+      type = tiles[i].type;
+      for (j = 0; j < MAX_TILES && !have_match; j++) {
+	if (tiles[j].type == type && i != j && tile_free (j)) {
+          have_match = TRUE;
+          hint_tiles[0] = i;
+          hint_tiles[1] = j;
+        }
+      }
+    }
+  }
+   
+  if (!have_match)
+    return;
+
+  /* Clear selection if no part of the hint */
+  if (selected_tile < MAX_TILES && hint_tiles[0] != selected_tile) {
+    tiles[selected_tile].selected &= ~SELECTED_FLAG;
     draw_tile (selected_tile);
     selected_tile = MAX_TILES + 1;
   }
 
-  for (i = 0; i < MAX_TILES && !free; i++)
-    if (tile_free (i)) {
-      type = tiles[i].type;
-      for (j = 0; j < MAX_TILES && !free; j++) {
-	free = (tiles[j].type == type && i != j && tile_free (j));
-	if (free) {
-	  tiles[i].selected ^= HINT_FLAG;
-	  tiles[j].selected ^= HINT_FLAG;
-	  draw_tile (i);
-	  draw_tile (j);
-	  hint_tiles[0] = i;
-	  hint_tiles[1] = j;
-	}
-      }
-    }
+  tiles[hint_tiles[0]].selected |= HINT_FLAG;
+  tiles[hint_tiles[1]].selected |= HINT_FLAG;
+  draw_tile (hint_tiles[0]);
+  draw_tile (hint_tiles[1]);
 
   /* This is a good way to test check_free
    * for (i=0;i<MAX_TILES;i++)
@@ -963,15 +984,6 @@ pause_callback (void)
     message ("");
   }
   noloops = FALSE;
-
-
-}
-
-void
-ensure_pause_off (void)
-{
-  if (paused)
-    pause_callback ();
 }
 
 static void
@@ -991,43 +1003,16 @@ scores_callback (GtkAction * action, gpointer data)
   gtk_widget_hide (dialog);
 }
 
-static void
-init_game (void)
-{
-  gchar *newtitle;
-
-  /* Translators: This is the window title for Mahjongg which contains the map name, e.g. 'Mahjongg - Red Dragon' */
-  newtitle = g_strdup_printf (_("Mahjongg - %s"), Q_ (maps[mapset].name));
-  gtk_window_set_title (GTK_WINDOW (window), newtitle);
-  g_free (newtitle);
-
-  score_current_mapset = maps[mapset].score_name;
-
-  gtk_label_set_text (GTK_LABEL (tiles_label), MAX_TILES_STR);
-  update_moves_left ();
-  game_over = GAME_WAITING;
-  sequence_number = 1;
-  visible_tiles = MAX_TILES;
-  selected_tile = MAX_TILES + 1;
-  set_undoredo_state (FALSE, FALSE);
-
-  games_clock_stop (GAMES_CLOCK (chrono));
-  games_clock_set_seconds (GAMES_CLOCK (chrono), 0);
-}
-
 static void
 new_game_cb (GtkAction * action, gpointer data)
 {
-  stop_hints ();
-  ensure_pause_off ();
-  new_game ();
+  new_game (TRUE);
 }
 
 static void
 restart_game_cb (GtkAction * action, gpointer data)
 {
-  stop_hints ();
-  restart_game ();
+  new_game (FALSE);
 }
 
 static void
@@ -1036,21 +1021,6 @@ quit_cb (GObject * object, gpointer data)
   gtk_main_quit ();
 }
 
-void
-restart_game (void)
-{
-  gint i;
-
-  ensure_pause_off ();
-  for (i = 0; i < MAX_TILES; i++) {
-    tiles[i].visible = 1;
-    tiles[i].selected = 0;
-    tiles[i].sequence = 0;
-  }
-  draw_all_tiles ();
-  init_game ();
-}
-
 static void
 redo_tile_callback (GtkAction * action, gpointer data)
 {
@@ -1094,6 +1064,7 @@ redo_tile_callback (GtkAction * action, gpointer data)
       break;
     }
   }
+
   set_undoredo_state (TRUE, found);
 
   update_moves_left ();
@@ -1149,25 +1120,6 @@ clear_undo_queue (void)
       tiles[lp].sequence = 0;
 }
 
-static void
-load_map (void)
-{
-  new_map = FALSE;
-  pos = maps[mapset].map;
-  generate_dependencies ();
-  calculate_view_geometry ();
-  configure_pixmaps ();
-}
-
-static void
-do_game (void)
-{
-  games_scores_set_category (highscores, maps[mapset].score_name);
-  if (new_map)
-    load_map ();
-  generate_game (g_random_int ());	/* puts in the positions of the tiles */
-}
-
 static void
 load_preferences (void)
 {
@@ -1185,20 +1137,71 @@ load_preferences (void)
 }
 
 void
-new_game (void)
+new_game (gboolean shuffle)
 {
-  do_game ();
+  gint i;
+  gchar *title;
 
+  /* Reset state */
+  game_over = GAME_WAITING;
+  sequence_number = 1;
+  visible_tiles = MAX_TILES;
+  selected_tile = MAX_TILES + 1;
+  undo_state = FALSE;
+  redo_state = FALSE;
+  hint_tiles[0] = MAX_TILES + 1;
+  hint_tiles[1] = MAX_TILES + 1;   
+  if (timer)
+    g_source_remove (timer);
+  timer = 0;
+  timeout_counter = HINT_BLINK_NUM + 1;
+  moves_left = 0;
+  paused = FALSE;
+  for (i = 0; i < MAX_TILES; i++) {
+    tiles[i].visible = 1;
+    tiles[i].selected = 0;
+    tiles[i].sequence = 0;
+  }
+
+  /* Load map */
+  if (active_mapset != mapset) {
+    active_mapset = mapset;
+    pos = maps[mapset].map;
+    generate_dependencies ();
+    calculate_view_geometry ();
+    configure_pixmaps ();
+    games_scores_set_category (highscores, maps[mapset].score_name);
+  }
+
+  /* Generate layout */
+  if (shuffle) {
+     generate_game (g_random_int ());
+  }
+   
+  /* Analyse layout */
+  update_moves_left ();
+  
+  /* Update menus */
+  update_menu_sensitivities ();
+
+  /* Set window title */
+  /* Translators: This is the window title for Mahjongg which contains the map name, e.g. 'Mahjongg - Red Dragon' */
+  title = g_strdup_printf (_("Mahjongg - %s"), Q_ (maps[mapset].name));
+  gtk_window_set_title (GTK_WINDOW (window), title);
+  g_free (title);
+  gtk_label_set_text (GTK_LABEL (tiles_label), MAX_TILES_STR);
+
+  /* Prepare clock */
+  games_clock_stop (GAMES_CLOCK (chrono));
+  games_clock_set_seconds (GAMES_CLOCK (chrono), 0);
+
+  /* Redraw */
   draw_all_tiles ();
-
-  init_game ();
 }
 
 void
 shuffle_tiles_callback (void)
 {
-  gboolean ok;
-
   if (paused || game_over == GAME_DEAD || game_over == GAME_WON)
     return;
 
@@ -1209,36 +1212,23 @@ shuffle_tiles_callback (void)
     unselect_tile (selected_tile);
   }
 
-  ok = shuffle ();
+  /* Shuffle the tiles - this should always succeed, the option should not be
+   * available if they cannot be shuffled */
+  if (!shuffle ())
+    return;
 
-  if (!ok) {
-    GtkWidget *mb;
-    game_over = GAME_DEAD;
-    games_clock_stop (GAMES_CLOCK (chrono));
-    mb = gtk_message_dialog_new (GTK_WINDOW (window),
-				 GTK_DIALOG_MODAL |
-				 GTK_DIALOG_DESTROY_WITH_PARENT,
-				 GTK_MESSAGE_ERROR, GTK_BUTTONS_OK,
-				 (_
-				  ("Sorry, I was unable to find a playable configuration.")));
-    gtk_dialog_run (GTK_DIALOG (mb));
-    gtk_widget_destroy (mb);
+  draw_all_tiles ();
 
-  } else {
+  game_over = GAME_RUNNING;
 
-    draw_all_tiles ();
+  /* 60s penalty */
+  games_clock_add_seconds (GAMES_CLOCK (chrono), 60);
 
-    game_over = GAME_RUNNING;
-
-    /* 60s penalty */
-    games_clock_add_seconds (GAMES_CLOCK (chrono), 60);
-
-    update_moves_left ();
-    /* Disable undo/redo after a shuffle. */
-    sequence_number = 1;
-    clear_undo_queue ();
-    set_undoredo_state (FALSE, FALSE);
-  }
+  update_moves_left ();
+  /* Disable undo/redo after a shuffle. */
+  sequence_number = 1;
+  clear_undo_queue ();
+  set_undoredo_state (FALSE, FALSE);
 
   update_menu_sensitivities ();
 }
@@ -1515,9 +1505,8 @@ main (int argc, char *argv[])
 
   /* FIXME: get these in the best place (as per the comment below. */
   init_config ();
-
-  do_game ();
-  init_game ();
+   
+  new_game (TRUE);
 
   /* Don't leave the keyboard focus on the toolbar */
   gtk_widget_grab_focus (board);
